package com.tropo.persist;

import java.util.*;
import java.io.*;
import jdbm.*;
import jdbm.btree.*;
import jdbm.helper.*;
import jdbm.recman.*;

import com.tropo.*;

/**
 * Persistent List.
 */
public final class PersistentList
	extends TRBase
	implements List
{
	/**
	 *
	 */
	public PersistentList( String fn)
		throws IOException
	{
		pm = new PersistentMap( fn, new LongComparator());
	}
	/**
	 *
	 */
    public int size()
	{
		return pm.size();
	}
	
	/**
	 *
	 */
    public boolean isEmpty()
	{
		return pm.size() == 0;
	}
	
	/**
	 *
	 */
    public boolean contains( Object x)
	{
		return pm.containsKey( x);
	}
	
	/**
	 *
	 */
    public java.util.Iterator iterator()
	{
		try
		{
			return new MyIterator();
		}
		catch( IOException ioe)
		{
			throw new RuntimeException( ioe);			
		}
	}
	
	/**
	 *
	 */
    public Object[] toArray()
	{
		int n = size();
		Object[] ar = new Object[ n];
		Iterator it = iterator();
		int index = 0;
		while ( it.hasNext() && index < n)
			ar[ index++] = it.next();
		return ar;
	}
	
	/**
	 *
	 */
    public Object toArray( Object x[])[]
	{
		nope();
		return null;		
	}
	
	/**
	 *
	 */
    public boolean add( Object x)
	{
		pm.put( new Long( next++), x);
		return true;
	}
	
	/**
	 *
	 */
    public boolean remove( Object x)
	{
		try
		{
			TupleBrowser tb = pm.bt.browse();
			Tuple cur = new Tuple();		
			while( tb.getNext( cur))
			{
				if ( cur.getValue().equals( x))
				{
					pm.remove( cur.getKey()); 
					return true; // removes first occurrance
				}
			}
			return false;
		}
		catch( IOException ioe)
		{
			return false;
		}
	}
	
	/**
	 *
	 */
    public boolean containsAll(Collection x)
	{
		nope();
		return false;
	}
	
	/**
	 *
	 */
    public boolean addAll(Collection x)
	{
		Iterator it = x.iterator();
		while ( it.hasNext())
			add( it.next());
		return true;
	}
	
	/**
	 *
	 */
    public boolean addAll(int y, Collection x)
	{
		unsup();
		return false;
	}
	
	/**
	 *
	 */
    public boolean removeAll(Collection x)
	{
		Iterator it = x.iterator();
		while ( it.hasNext())
			remove( it.next());		 // inefficient

		return true;
	}
	
	/**
	 *
	 */
    public boolean retainAll(Collection x)
	{
		nope();
		return false;
	}
	
	/**
	 *
	 */
    public void clear()
	{
		pm.clear();
	}
	
	/**
	 *
	 */
    public boolean equals( Object x)
	{
		nope();
		return false;
	}
	
	/**
	 *
	 */
    public int hashCode()
	{
		nope();
		return 0;
			
	}
	
	/**
	 *
	 */
    public Object get(int x)
	{
		try
		{
			TupleBrowser tb = pm.bt.browse();
			Tuple cur = new Tuple();
			while( tb.getNext( cur))
			{
				if ( x-- <= 0)
					return cur.getValue();
			}
			return null;
		}
		catch( IOException ioe)
		{
			return null;
		}

	}
	
	/**
	 *
	 */
    public Object set(int y,  Object x)
	{
		unsup();
		return null;
	}
	
	/**
	 *
	 */
    public void add(int y,  Object x)
	{
		unsup();
	}
	
	/**
	 *
	 */
    public Object remove(int y)
	{
		try
		{
			TupleBrowser tb = pm.bt.browse();
			Tuple cur = new Tuple();
			while( tb.getNext( cur))
			{
				if ( y-- <= 0)
				{
					pm.remove( cur.getKey());
					return cur.getValue();
				}
			}		
			return null;
		}
		catch( IOException ioe)
		{
			return null;
		}
	}
	
	/**
	 *
	 */
    public int indexOf( Object x)
	{
		try
		{
			TupleBrowser tb = pm.bt.browse();
			Tuple cur = new Tuple();
			int row = 0;
			while( tb.getNext( cur))
			{
				if ( cur.getValue().equals( x))
					return row;
				row++;
			}
			return -1;
		}
		catch( IOException ioe)
		{
			return -1;
		}
	}
	
	/**
	 *
	 */
    public int lastIndexOf( Object x)
	{
		nope();
		return 0;
	}
	
	/**
	 *
	 */
    public ListIterator listIterator()
	{
		nope();
		return null;
	}
	
	/**
	 *
	 */
    public ListIterator listIterator(int skip)
	{
		try
		{
			return new MyListIterator( skip);
		}
		catch( IOException ioe)
		{
			throw new RuntimeException( ioe);						
		}
	}
	
	/**
	 *
	 */
    public List subList(int x, int y)
	{
		nope();
		return null;		
	}
	
	/**
	 *
	 */	
	private void nope()
	{
		new Throwable().printStackTrace();
		System.out.println( "NOT IMPL");
		System.exit( 1);
	}
	/**
	 *
	 */
	private void unsup()
	{
		throw new UnsupportedOperationException();
	}

	/**
	 *
	 */
	public static void main( String [] a)
		throws Throwable
	{
		new File( "foo.db").delete();
		new File( "foo.lg").delete();		
		List lis = new PersistentList( "foo");
		int n = 0;
		for ( int i = 0; i < 1000; i++)
			for ( int j = 0; j < i; j++)
			{
				lis.add( ""+ i + "-" + j);
				if ( n++ % 1000 == 0)
					o.println( "" + i + "/" + j + " / " + lis.size());
			}
		o.println( "before it");
		Iterator it = lis.iterator();
		o.println( "after it");		
		int num = 0;
		while ( it.hasNext())
		{
			it.next();
			num++;
			//o.println( "next: " + it.next());
		}
		o.println( "num=" +num +" size=" + lis.size());
	}
	private PersistentMap pm;

	private long next;

	/*
	private static class Ent
	{
		Ent( Object key, Object val)
	}
	*/
	
	/**
	 *
	 */
	private class MyIterator
		implements Iterator
	{
		/**
		 *
		 */
		private MyIterator()
			throws IOException
		{
			tb = pm.bt.browse();
		}
			 
		/**
		 *
		 */
		public boolean hasNext()
		{
			try
			{
				return tb.getNext( cur);
			}
			catch( IOException ioe)
			{
				throw new RuntimeException( ioe);
			}
		}

		/**
		 *
		 */
		public Object next()
		{
			return cur.getValue();
		}

		/**
		 *
		 */
		public void remove()
		{
			throw new UnsupportedOperationException();
		}
		private TupleBrowser tb;
		private Tuple cur = new Tuple();
	}

	/**
	 *
	 */
	private class MyListIterator
		implements ListIterator
	{
		/**
		 *
		 */
		private MyListIterator( int skip)
			throws IOException
		{
			tb = pm.bt.browse();
			for ( int i = 0; i< skip; i++)
				tb.getNext( cur);
		}

		/**
		 *
		 */
		public boolean hasPrevious()
		{
			nope();
			return false;
		}

		/**
		 *
		 */		
		public java.lang.Object previous()
		{
			row--;
			nope();
			return null;
		}

		/**
		 *
		 */
		public int nextIndex()
		{
			return row+1;
		}

		/**
		 *
		 */		
		public int previousIndex()
		{
			return row-1;
		}


		/**
		 *
		 */		
		public void set(java.lang.Object x)
		{
			nope();			
		}

		/**
		 *
		 */		
		public void add(java.lang.Object x)
		{
			nope();
		}
		
		/**
		 *
		 */
		public boolean hasNext()
		{
			try
			{
				return tb.getNext( cur);
			}
			catch( IOException ioe)
			{
				throw new RuntimeException( ioe);
			}
		}

		/**
		 *
		 */
		public Object next()
		{
			row++;
			return cur.getValue();
		}

		/**
		 *
		 */
		public void remove()
		{
			throw new UnsupportedOperationException();
		}

	    private int row;
		private TupleBrowser tb;
		private Tuple cur = new Tuple();
	}
}
