package com.tropo.persist;

import com.tropo.persist.*;
import java.util.*;
import junit.framework.*;

public class PersistentMapTestCase
	extends TestCase
{
	/**
	 *
	 */
	public PersistentMapTestCase( String name)
	{
		super( name);
	}

	/**
	 *
	 */
	public PersistentMapTestCase()
	{
	}
	
	/**
	 *
	 */
	protected void setUp()
		throws Exception
	{
		pm = new PersistentMap( fn); 
	}

	/**
	 *
	 */
	public void test1()
		throws Throwable		
	{
		pm.put( "a", "b");
		assertTrue( pm.containsKey( "a"));
	}

	/**
	 *
	 */
	public void test2()
		throws Throwable		
	{
		int before = pm.size();
		pm.put( "c", "d");
		pm.put( "c", "e"); 
		assertTrue( pm.containsKey( "c"));
		assertTrue( pm.size() == (1+before));
	}
	
	/**
	 *
	 */
	public void test3() 
		throws Throwable		
	{
		clear();
		Set s = pm.keySet();
		assertTrue( s.size() == 0);
	}
	/**
	 *
	 */
	public void test4() 
		throws Throwable		
	{
		clear();

		for ( int i = 0; i < 100; i++)
		{
			String key = "" + i;
			byte[] value = new byte[ i];
			for ( int j = 0; j < i; j++)
				value[ j] = (byte) i; // constant
			
			pm.put( key, value);
		}
		for ( int i = 0; i < 100; i++)
		{
			String key = "" + i;
			byte[] value = (byte[]) pm.get( key);
			assertTrue( value != null);
			assertTrue( value.length == i);
			for ( int j = 0; j < i; j++)
				assertTrue( value[ j] == i);
		}
	}

	/**
	 * Vary data sizes.
	 */
	public void test5() 
		throws Throwable		
	{
		clear();

		for ( int i = 0; i < 100; i++)
		{
			String key = "" + i;
			byte[] value = new byte[ i];
			for ( int j = 0; j < i; j++)
				value[ j] = (byte) i; // constant
			
			pm.put( key, value);
		}
		for ( int i = 0; i < 100; i++)
		{
			String key = "" + i;
			int len = i * 100;
			byte[] value = new byte[ len];
			for ( int j = 0; j < len; j++)
				value[ j] = (byte) i; // constant
			
			pm.put( key, value);
		}		
		for ( int i = 0; i < 100; i++)
		{
			String key = "" + i;
			byte[] value = (byte[]) pm.get( key);
			int len = i * 100;			
			assertTrue( value != null);
			assertTrue( value.length == len);
			for ( int j = 0; j < len; j++)
				assertTrue( value[ j] == i);
		}
	}

	/**
	 * Vary key sizes. Slow.
	 */
	public void test99() 
		throws Throwable		
	{
		clear();
		assertTrue( pm.remove( "foo") == null);
		
		System.out.println( "test99, phase 1");
		for ( int i = 0; i < 100; i++)
		{
			String key = "" + i;
			byte[] value = new byte[ i];
			for ( int j = 0; j < i; j++)
				value[ j] = (byte) i; // constant
			
			pm.put( key, value);
		}
		System.out.println( "test99, phase 2");		
		for ( int i = 0; i < 100; i++)
		{
			if ( (i % 10) == 0)
				System.out.println( "test99, phase 2."+ i);
			String key = "" + i;
			int len = i * 100;
			for ( int j = 0; j < len; j++)
				key += "x";
			
			byte[] value = new byte[ len];
			for ( int j = 0; j < len; j++)
				value[ j] = (byte) i; // constant
			
			pm.put( key, value);
		}
		System.out.println( "test99, phase 3");		
		for ( int i = 0; i < 100; i++)
		{
			if ( (i % 10) == 0)
				System.out.println( "test99, phase 3."+ i); 
			String key = "" + i;
			int len = i * 100;
			for ( int j = 0; j < len; j++)
				key += "x";			
			byte[] value = (byte[]) pm.get( key);

			assertTrue( value != null);
			assertTrue( value.length == len);
			for ( int j = 0; j < len; j++)
				assertTrue( value[ j] == i);
		}
	}

	/**
	 * Create then delete.
	 */
	public void test7() 
		throws Throwable		
	{
		clear();

		for ( int i = 0; i < 100; i++)
		{
			String key = "" + i;
			pm.put( key, key);
			assertEquals( "Size after insert", pm.size(), (1+i));
			assertTrue( "has key", pm.containsKey( key));
		}
		for ( int i = 0; i < 100; i++)
		{
			String key = "" + i;
			assertTrue( "remove returns key", pm.remove( key) != null);
			assertTrue( "no key after remove", ! pm.containsKey( key));			
			assertEquals( "right size after rm", pm.size(), (100-(i+1)));
		}
	}

	/**
	 * Iterator w/ parallel map.
	 */
	public void test8() 
		throws Throwable		
	{
		clear();
		Map m = new HashMap();
		for ( int i = 0; i < 100; i++)
		{
			String key = "" + i;
			byte[] value = new byte[ i];
			for ( int j = 0; j < i; j++)
				value[ j] = (byte) i; // constant
			if ( (i % 7) == 0)
				key = ('a'  + (i%7)) + key;
			
			pm.put( key, value);
			m.put( key, value);
		}
		Set s1 = pm.keySet();
		Set s2 = m.keySet();
		assertTrue( s1.size() == 100);
		assertTrue( s2.size() == 100);
		Set keys = new HashSet();		
		Iterator x1 = s1.iterator();
		while ( x1.hasNext())
			keys.add( x1.next());
		// remove now
		Iterator x2 = s2.iterator();
		while ( x2.hasNext())
		{
			Object z = x2.next();
			assertTrue( keys.contains( z));
			keys.remove( z);
		}
		assertTrue( keys.size() == 0);
		

	}

	/**
	 * Create then delete.
	 */
	private void clear()
		throws Throwable		
	{
		pm.clear();
		assertEquals( "size 0 after clear()", pm.size(), 0);
		assertTrue( "emtpy after clear", pm.isEmpty());
	}

	/**
	 *
	 */
	protected void tearDown()
		throws Exception
	{
		try
		{
			pm.destroy();
		}
		catch( Throwable t)
		{
			throw new Exception( t.toString());
		}
	}
	String fn = "foo";
	PersistentMap pm;

}
